#include "CTerrain.h"
#include <fstream.h>
#include <windows.h>
#include <iostream.h>
#include <gl.h>
#include <glaux.h>
#include "CTexture.h"


//----------------------------------------------------------------------------------------
// CTerrain()
//----------------------------------------------------------------------------------------
CTerrain::CTerrain()
{
	this->Texture = new CTexture();
	this->Texture->Load("Resources//Textures//Green.bmp");
}


//----------------------------------------------------------------------------------------
// ~CTerrain()
//----------------------------------------------------------------------------------------
CTerrain::~CTerrain()
{
	if(HeightMap)
		delete [] HeightMap;
}


//----------------------------------------------------------------------------------------
// LoadRAWGrayScale()
// Desc: Loads .RAW file, and puts height field data into HeightMap[]
//----------------------------------------------------------------------------------------
int CTerrain::LoadRAWGrayScale(char* filename)
{
	ifstream RAWFile;
	RAWFile.open(filename, ios::binary| ios::nocreate, NULL);

	if(!RAWFile.is_open())
	{
	//	char* error_string = "Could not open .RAW GrayScale file: ";
	//	strncat(error_string, filename, sizeof(filename));
		MessageBox(NULL, "Could not open .RAW file", "ERROR", MB_OK);
	}

	RAWFile.read(HeightMap, this->size*this->size);

	RAWFile.close();

 return 0;

}


//----------------------------------------------------------------------------------------
// Build()
// Desc: Generates the heightfield map, using the 2d grayscale bitmap, and fills in all
// the vertex arrays.
//----------------------------------------------------------------------------------------
int CTerrain::Build(char* filename, int a_size, int a_smoothness)
{
	this->size				= a_size;
	this->smoothness		= a_smoothness;

	this->HeightMap = new unsigned char[this->size*this->size];

	LoadRAWGrayScale(filename);

 return 0;
}


//----------------------------------------------------------------------------------------
// SetVertexColor()
// Desc:
//----------------------------------------------------------------------------------------
int CTerrain::SetVertexColor(int X, int Z)
{
		float fColor = -0.15f + (HeightMap[Z*size + X] / 256.0f);
		glColor3f(0.0f, fColor, 0.0f  );		// Assign This Green Shade To The Current Vertex

 return 0;
}


//----------------------------------------------------------------------------------------
// Draw()
// Desc: Render terrain
//----------------------------------------------------------------------------------------
int CTerrain::Render(void)
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, this->Texture->GetID());
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glBegin(GL_QUADS);
		for(int x=0; x<size; x+= smoothness)
		{
			for(int z=0; z<size; z+= smoothness)
			{
				glTexCoord2d(0.0f, 0.0f); SetVertexColor(x, z);
				glVertex3i(x, HeightMap[z*size + x], z);
					
				glTexCoord2d(0.0f ,1.0f); SetVertexColor(x, z + smoothness);
				glVertex3i(x, HeightMap[(z+smoothness)*size + x], z + smoothness); 
					
				glTexCoord2d(1.0f, 1.0f); SetVertexColor(x + smoothness, z + smoothness);
				glVertex3i(x + smoothness, HeightMap[(z+smoothness)*size + x + smoothness], z + smoothness); 
					
				glTexCoord2d(1.0f, 0.0f); SetVertexColor(x + smoothness, z);
				glVertex3i(x + smoothness, HeightMap[z*size + x + smoothness], z); 
			}
		}
	glEnd();
	glDisable(GL_TEXTURE_2D);

 return 0;
}
